@@ -1,6 +1,7 @@
 """
 pytest PluginManager, basic initialization and tracing.
 """
+import os
 import sys
 import inspect
 import py
@@ -9,6 +10,8 @@ import py
 assert py.__version__.split(".")[:2] >= ['1', '4'], ("installation problem: "
     "%s is too old, remove or upgrade 'py'" % (py.__version__))
 
+py3 = sys.version_info > (3,0)
+
 class TagTracer:
     def __init__(self):
         self._tag2proc = {}
@@ -66,16 +69,104 @@ class TagTracerSub:
     def get(self, name):
         return self.__class__(self.root, self.tags + (name,))
 
+
+def add_method_wrapper(cls, wrapper_func):
+    """ Substitute the function named "wrapperfunc.__name__" at class
+    "cls" with a function that wraps the call to the original function.
+    Return an undo function which can be called to reset the class to use
+    the old method again.
+
+    wrapper_func is called with the same arguments as the method
+    it wraps and its result is used as a wrap_controller for
+    calling the original function.
+    """
+    name = wrapper_func.__name__
+    oldcall = getattr(cls, name)
+    def wrap_exec(*args, **kwargs):
+        gen = wrapper_func(*args, **kwargs)
+        return wrapped_call(gen, lambda: oldcall(*args, **kwargs))
+
+    setattr(cls, name, wrap_exec)
+    return lambda: setattr(cls, name, oldcall)
+
+def raise_wrapfail(wrap_controller, msg):
+    co = wrap_controller.gi_code
+    raise RuntimeError("wrap_controller at %r %s:%d %s" %
+                   (co.co_name, co.co_filename, co.co_firstlineno, msg))
+
+def wrapped_call(wrap_controller, func):
+    """ Wrap calling to a function with a generator which needs to yield
+    exactly once.  The yield point will trigger calling the wrapped function
+    and return its CallOutcome to the yield point.  The generator then needs
+    to finish (raise StopIteration) in order for the wrapped call to complete.
+    """
+    try:
+        next(wrap_controller)   # first yield
+    except StopIteration:
+        raise_wrapfail(wrap_controller, "did not yield")
+    call_outcome = CallOutcome(func)
+    try:
+        wrap_controller.send(call_outcome)
+        raise_wrapfail(wrap_controller, "has second yield")
+    except StopIteration:
+        pass
+    return call_outcome.get_result()
+
+
+class CallOutcome:
+    """ Outcome of a function call, either an exception or a proper result.
+    Calling the ``get_result`` method will return the result or reraise
+    the exception raised when the function was called. """
+    excinfo = None
+    def __init__(self, func):
+        try:
+            self.result = func()
+        except BaseException:
+            self.excinfo = sys.exc_info()
+
+    def force_result(self, result):
+        self.result = result
+        self.excinfo = None
+
+    def get_result(self):
+        if self.excinfo is None:
+            return self.result
+        else:
+            ex = self.excinfo
+            if py3:
+                raise ex[1].with_traceback(ex[2])
+            py.builtin._reraise(*ex)
+
+
 class PluginManager(object):
-    def __init__(self, hookspecs=None):
+    def __init__(self, hookspecs=None, prefix="pytest_"):
         self._name2plugin = {}
-        self._listattrcache = {}
         self._plugins = []
+        self._conftestplugins = []
+        self._plugin2hookcallers = {}
         self._warnings = []
         self.trace = TagTracer().get("pluginmanage")
         self._plugin_distinfo = []
         self._shutdown = []
-        self.hook = HookRelay(hookspecs or [], pm=self)
+        self.hook = HookRelay(hookspecs or [], pm=self, prefix=prefix)
+
+    def set_tracing(self, writer):
+        self.trace.root.setwriter(writer)
+        # reconfigure HookCalling to perform tracing
+        assert not hasattr(self, "_wrapping")
+        self._wrapping = True
+
+        def _docall(self, methods, kwargs):
+            trace = self.hookrelay.trace
+            trace.root.indent += 1
+            trace(self.name, kwargs)
+            box = yield
+            if box.excinfo is None:
+                trace("finish", self.name, "-->", box.result)
+            trace.root.indent -= 1
+
+        undo = add_method_wrapper(HookCaller, _docall)
+        self.add_shutdown(undo)
 
     def do_configure(self, config):
         # backward compatibility
@@ -85,7 +176,7 @@ class PluginManager(object):
         assert not hasattr(self, "_registercallback")
         self._registercallback = callback
 
-    def register(self, plugin, name=None, prepend=False):
+    def register(self, plugin, name=None, prepend=False, conftest=False):
         if self._name2plugin.get(name, None) == -1:
             return
         name = name or getattr(plugin, '__name__', str(id(plugin)))
@@ -93,23 +184,35 @@ class PluginManager(object):
             raise ValueError("Plugin already registered: %s=%s\n%s" %(
                               name, plugin, self._name2plugin))
         #self.trace("registering", name, plugin)
-        self._name2plugin[name] = plugin
         reg = getattr(self, "_registercallback", None)
         if reg is not None:
-            reg(plugin, name)
-        if not prepend:
-            self._plugins.append(plugin)
+            reg(plugin, name)  # may call addhooks
+        hookcallers = list(self.hook._scan_plugin(plugin))
+        self._plugin2hookcallers[plugin] = hookcallers
+        self._name2plugin[name] = plugin
+        if conftest:
+            self._conftestplugins.append(plugin)
         else:
-            self._plugins.insert(0, plugin)
+            if not prepend:
+                self._plugins.append(plugin)
+            else:
+                self._plugins.insert(0, plugin)
+        # finally make sure that the methods of the new plugin take part
+        for hookcaller in hookcallers:
+            hookcaller.scan_methods()
         return True
 
-    def unregister(self, plugin=None, name=None):
-        if plugin is None:
-            plugin = self.getplugin(name=name)
-        self._plugins.remove(plugin)
+    def unregister(self, plugin):
+        try:
+            self._plugins.remove(plugin)
+        except KeyError:
+            self._conftestplugins.remove(plugin)
         for name, value in list(self._name2plugin.items()):
             if value == plugin:
                 del self._name2plugin[name]
+        hookcallers = self._plugin2hookcallers.pop(plugin)
+        for hookcaller in hookcallers:
+            hookcaller.scan_methods()
 
     def add_shutdown(self, func):
         self._shutdown.append(func)
@@ -118,22 +221,19 @@ class PluginManager(object):
         while self._shutdown:
             func = self._shutdown.pop()
             func()
-        self._plugins = []
+        self._plugins = self._conftestplugins = []
         self._name2plugin.clear()
-        self._listattrcache.clear()
 
     def isregistered(self, plugin, name=None):
         if self.getplugin(name) is not None:
             return True
-        for val in self._name2plugin.values():
-            if plugin == val:
-                return True
+        return plugin in self._plugins or plugin in self._conftestplugins
 
     def addhooks(self, spec, prefix="pytest_"):
         self.hook._addhooks(spec, prefix=prefix)
 
     def getplugins(self):
-        return list(self._plugins)
+        return self._plugins + self._conftestplugins
 
     def skipifmissing(self, name):
         if not self.hasplugin(name):
@@ -154,7 +254,7 @@ class PluginManager(object):
     # API for bootstrapping
     #
     def _envlist(self, varname):
-        val = py.std.os.environ.get(varname, None)
+        val = os.environ.get(varname, None)
         if val is not None:
             return val.split(',')
         return ()
@@ -189,15 +289,17 @@ class PluginManager(object):
     def consider_pluginarg(self, arg):
         if arg.startswith("no:"):
             name = arg[3:]
-            if self.getplugin(name) is not None:
-                self.unregister(None, name=name)
+            plugin = self.getplugin(name)
+            if plugin is not None:
+                self.unregister(plugin)
             self._name2plugin[name] = -1
         else:
             if self.getplugin(arg) is None:
                 self.import_plugin(arg)
 
     def consider_conftest(self, conftestmodule):
-        if self.register(conftestmodule, name=conftestmodule.__file__):
+        if self.register(conftestmodule, name=conftestmodule.__file__,
+                         conftest=True):
             self.consider_module(conftestmodule)
 
     def consider_module(self, mod):
@@ -221,7 +323,7 @@ class PluginManager(object):
                 return self.import_plugin(modname[7:])
             raise
         except:
-            e = py.std.sys.exc_info()[1]
+            e = sys.exc_info()[1]
             import pytest
             if not hasattr(pytest, 'skip') or not isinstance(e, pytest.skip.Exception):
                 raise
@@ -232,12 +334,7 @@ class PluginManager(object):
 
     def listattr(self, attrname, plugins=None):
         if plugins is None:
-            plugins = self._plugins
-        key = (attrname,) + tuple(plugins)
-        try:
-            return list(self._listattrcache[key])
-        except KeyError:
-            pass
+            plugins = self._plugins + self._conftestplugins
         l = []
         last = []
         wrappers = []
@@ -256,7 +353,6 @@ class PluginManager(object):
                 l.append(meth)
         l.extend(last)
         l.extend(wrappers)
-        self._listattrcache[key] = list(l)
         return l
 
     def call_plugin(self, plugin, methname, kwargs):
@@ -277,16 +373,10 @@ def importplugin(importspec):
 class MultiCall:
     """ execute a call into multiple python functions/methods. """
 
-    class WrongHookWrapper(Exception):
-        """ a hook wrapper does not behave correctly. """
-        def __init__(self, func, message):
-            Exception.__init__(self, func, message)
-            self.func = func
-            self.message = message
-
     def __init__(self, methods, kwargs, firstresult=False):
         self.methods = list(methods)
         self.kwargs = kwargs
+        self.kwargs["__multicall__"] = self
         self.results = []
         self.firstresult = firstresult
 
@@ -295,51 +385,22 @@ class MultiCall:
         return "<MultiCall %s, kwargs=%r>" %(status, self.kwargs)
 
     def execute(self):
-        next_finalizers = []
-        try:
-            while self.methods:
-                method = self.methods.pop()
-                kwargs = self.getkwargs(method)
-                if hasattr(method, "hookwrapper"):
-                    it = method(**kwargs)
-                    next = getattr(it, "next", None)
-                    if next is None:
-                        next = getattr(it, "__next__", None)
-                        if next is None:
-                            raise self.WrongHookWrapper(method,
-                                "wrapper does not contain a yield")
-                    res = next()
-                    next_finalizers.append((method, next))
-                else:
-                    res = method(**kwargs)
-                if res is not None:
-                    self.results.append(res)
-                    if self.firstresult:
-                        return res
-            if not self.firstresult:
-                return self.results
-        finally:
-            for method, fin in reversed(next_finalizers):
-                try:
-                    fin()
-                except StopIteration:
-                    pass
-                else:
-                    raise self.WrongHookWrapper(method,
-                                "wrapper contain more than one yield")
+        all_kwargs = self.kwargs
+        while self.methods:
+            method = self.methods.pop()
+            args = [all_kwargs[argname] for argname in varnames(method)]
+            if hasattr(method, "hookwrapper"):
+                return wrapped_call(method(*args), self.execute)
+            res = method(*args)
+            if res is not None:
+                self.results.append(res)
+                if self.firstresult:
+                    return res
+        if not self.firstresult:
+            return self.results
 
 
-    def getkwargs(self, method):
-        kwargs = {}
-        for argname in varnames(method):
-            try:
-                kwargs[argname] = self.kwargs[argname]
-            except KeyError:
-                if argname == "__multicall__":
-                    kwargs[argname] = self
-        return kwargs
-
-def varnames(func):
+def varnames(func, startindex=None):
     """ return argument name tuple for a function, method, class or callable.
 
     In case of a class, its "__init__" method is considered.
@@ -356,74 +417,127 @@ def varnames(func):
             func = func.__init__
         except AttributeError:
             return ()
-        ismethod = True
+        startindex = 1
     else:
         if not inspect.isfunction(func) and not inspect.ismethod(func):
             func = getattr(func, '__call__', func)
-        ismethod = inspect.ismethod(func)
+        if startindex is None:
+            startindex = int(inspect.ismethod(func))
+
     rawcode = py.code.getrawcode(func)
     try:
-        x = rawcode.co_varnames[ismethod:rawcode.co_argcount]
+        x = rawcode.co_varnames[startindex:rawcode.co_argcount]
     except AttributeError:
         x = ()
+    else:
+        defaults = func.__defaults__
+        if defaults:
+            x = x[:-len(defaults)]
     try:
         cache["_varnames"] = x
     except TypeError:
         pass
     return x
 
+
 class HookRelay:
     def __init__(self, hookspecs, pm, prefix="pytest_"):
         if not isinstance(hookspecs, list):
             hookspecs = [hookspecs]
-        self._hookspecs = []
         self._pm = pm
         self.trace = pm.trace.root.get("hook")
+        self.prefix = prefix
         for hookspec in hookspecs:
             self._addhooks(hookspec, prefix)
 
-    def _addhooks(self, hookspecs, prefix):
-        self._hookspecs.append(hookspecs)
+    def _addhooks(self, hookspec, prefix):
         added = False
-        for name, method in vars(hookspecs).items():
+        isclass = int(inspect.isclass(hookspec))
+        for name, method in vars(hookspec).items():
             if name.startswith(prefix):
                 firstresult = getattr(method, 'firstresult', False)
-                hc = HookCaller(self, name, firstresult=firstresult)
+                hc = HookCaller(self, name, firstresult=firstresult,
+                                argnames=varnames(method, startindex=isclass))
                 setattr(self, name, hc)
                 added = True
                 #print ("setting new hook", name)
         if not added:
             raise ValueError("did not find new %r hooks in %r" %(
-                prefix, hookspecs,))
+                prefix, hookspec,))
+
+    def _getcaller(self, name, plugins):
+        caller = getattr(self, name)
+        methods = self._pm.listattr(name, plugins=plugins)
+        if methods:
+            return caller.new_cached_caller(methods)
+        return caller
+
+    def _scan_plugin(self, plugin):
+        def fail(msg, *args):
+            name = getattr(plugin, '__name__', plugin)
+            raise PluginValidationError("plugin %r\n%s" %(name, msg % args))
+
+        for name in dir(plugin):
+            if not name.startswith(self.prefix):
+                continue
+            hook = getattr(self, name, None)
+            method = getattr(plugin, name)
+            if hook is None:
+                is_optional = getattr(method, 'optionalhook', False)
+                if not isgenerichook(name) and not is_optional:
+                    fail("found unknown hook: %r", name)
+                continue
+            for arg in varnames(method):
+                if arg not in hook.argnames:
+                    fail("argument %r not available\n"
+                         "actual definition: %s\n"
+                         "available hookargs: %s",
+                         arg, formatdef(method),
+                           ", ".join(hook.argnames))
+            yield hook
 
 
 class HookCaller:
-    def __init__(self, hookrelay, name, firstresult):
+    def __init__(self, hookrelay, name, firstresult, argnames, methods=()):
         self.hookrelay = hookrelay
         self.name = name
         self.firstresult = firstresult
-        self.trace = self.hookrelay.trace
+        self.argnames = ["__multicall__"]
+        self.argnames.extend(argnames)
+        assert "self" not in argnames  # sanity check
+        self.methods = methods
+
+    def new_cached_caller(self, methods):
+        return HookCaller(self.hookrelay, self.name, self.firstresult,
+                          argnames=self.argnames, methods=methods)
 
     def __repr__(self):
         return "<HookCaller %r>" %(self.name,)
 
-    def __call__(self, **kwargs):
-        methods = self.hookrelay._pm.listattr(self.name)
-        return self._docall(methods, kwargs)
+    def scan_methods(self):
+        self.methods = self.hookrelay._pm.listattr(self.name)
 
-    def pcall(self, plugins, **kwargs):
-        methods = self.hookrelay._pm.listattr(self.name, plugins=plugins)
-        return self._docall(methods, kwargs)
+    def __call__(self, **kwargs):
+        return self._docall(self.methods, kwargs)
+
+    def callextra(self, methods, **kwargs):
+        return self._docall(self.methods + methods, kwargs)
 
     def _docall(self, methods, kwargs):
-        self.trace(self.name, kwargs)
-        self.trace.root.indent += 1
-        mc = MultiCall(methods, kwargs, firstresult=self.firstresult)
-        try:
-            res = mc.execute()
-            if res:
-                self.trace("finish", self.name, "-->", res)
-        finally:
-            self.trace.root.indent -= 1
-        return res
+        return MultiCall(methods, kwargs,
+                         firstresult=self.firstresult).execute()
+
+
+class PluginValidationError(Exception):
+    """ plugin failed validation. """
+
+def isgenerichook(name):
+    return name == "pytest_plugins" or \
+           name.startswith("pytest_funcarg__")
+
+def formatdef(func):
+    return "%s%s" % (
+        func.__name__,
+        inspect.formatargspec(*inspect.getargspec(func))
+    )
 
