@@ -393,14 +393,31 @@ class TestFunction:
                 return 'value'
 
             @pytest.mark.parametrize('value',
-                                     ['overrided'])
-            def test_overrided_via_param(value):
-                assert value == 'overrided'
+                                     ['overridden'])
+            def test_overridden_via_param(value):
+                assert value == 'overridden'
         """)
         rec = testdir.inline_run()
         rec.assertoutcome(passed=1)
 
 
+    def test_parametrize_overrides_parametrized_fixture(self, testdir):
+        """Test parametrization when parameter overrides existing parametrized fixture with same name."""
+        testdir.makepyfile("""
+            import pytest
+
+            @pytest.fixture(params=[1, 2])
+            def value(request):
+                return request.param
+
+            @pytest.mark.parametrize('value',
+                                     ['overridden'])
+            def test_overridden_via_param(value):
+                assert value == 'overridden'
+        """)
+        rec = testdir.inline_run()
+        rec.assertoutcome(passed=1)
+
     def test_parametrize_with_mark(selfself, testdir):
         items = testdir.getitems("""
             import pytest
@@ -525,12 +542,15 @@ class TestConftestCustomization:
     def test_customized_pymakeitem(self, testdir):
         b = testdir.mkdir("a").mkdir("b")
         b.join("conftest.py").write(py.code.Source("""
-            def pytest_pycollect_makeitem(__multicall__):
-                result = __multicall__.execute()
-                if result:
-                    for func in result:
-                        func._some123 = "world"
-                return result
+            import pytest
+            @pytest.mark.hookwrapper
+            def pytest_pycollect_makeitem():
+                outcome = yield
+                if outcome.excinfo is None:
+                    result = outcome.result
+                    if result:
+                        for func in result:
+                            func._some123 = "world"
         """))
         b.join("test_module.py").write(py.code.Source("""
             import pytest
