@@ -46,6 +46,7 @@ class TestRaises:
                     1/0
                 print (excinfo)
                 assert excinfo.type == ZeroDivisionError
+                assert isinstance(excinfo.value, ZeroDivisionError)
 
             def test_noraise():
                 with pytest.raises(pytest.raises.Exception):
@@ -62,3 +63,10 @@ class TestRaises:
             '*3 passed*',
         ])
 
+    def test_noclass(self):
+        with pytest.raises(TypeError):
+            pytest.raises('wrong', lambda: None)
+
+    def test_tuple(self):
+        with pytest.raises((KeyError, ValueError)):
+            raise KeyError('oops')
