@@ -334,16 +334,16 @@ class TestSession:
         assert item.name == "test_func"
         newid = item.nodeid
         assert newid == id
-        py.std.pprint.pprint(hookrec.hookrecorder.calls)
+        py.std.pprint.pprint(hookrec.calls)
         topdir = testdir.tmpdir  # noqa
-        hookrec.hookrecorder.contains([
+        hookrec.assert_contains([
             ("pytest_collectstart", "collector.fspath == topdir"),
             ("pytest_make_collect_report", "collector.fspath == topdir"),
             ("pytest_collectstart", "collector.fspath == p"),
             ("pytest_make_collect_report", "collector.fspath == p"),
             ("pytest_pycollect_makeitem", "name == 'test_func'"),
             ("pytest_collectreport", "report.nodeid.startswith(p.basename)"),
-            ("pytest_collectreport", "report.nodeid == '.'")
+            ("pytest_collectreport", "report.nodeid == ''")
         ])
 
     def test_collect_protocol_method(self, testdir):
@@ -381,9 +381,9 @@ class TestSession:
         id = p.basename
 
         items, hookrec = testdir.inline_genitems(id)
-        py.std.pprint.pprint(hookrec.hookrecorder.calls)
+        py.std.pprint.pprint(hookrec.calls)
         assert len(items) == 2
-        hookrec.hookrecorder.contains([
+        hookrec.assert_contains([
             ("pytest_collectstart",
                 "collector.fspath == collector.session.fspath"),
             ("pytest_collectstart",
@@ -404,8 +404,8 @@ class TestSession:
 
         items, hookrec = testdir.inline_genitems()
         assert len(items) == 1
-        py.std.pprint.pprint(hookrec.hookrecorder.calls)
-        hookrec.hookrecorder.contains([
+        py.std.pprint.pprint(hookrec.calls)
+        hookrec.assert_contains([
             ("pytest_collectstart", "collector.fspath == test_aaa"),
             ("pytest_pycollect_makeitem", "name == 'test_func'"),
             ("pytest_collectreport",
@@ -425,8 +425,8 @@ class TestSession:
 
         items, hookrec = testdir.inline_genitems(id)
         assert len(items) == 2
-        py.std.pprint.pprint(hookrec.hookrecorder.calls)
-        hookrec.hookrecorder.contains([
+        py.std.pprint.pprint(hookrec.calls)
+        hookrec.assert_contains([
             ("pytest_collectstart", "collector.fspath == test_aaa"),
             ("pytest_pycollect_makeitem", "name == 'test_func'"),
             ("pytest_collectreport", "report.nodeid == 'aaa/test_aaa.py'"),
@@ -478,7 +478,7 @@ class Test_getinitialnodes:
         config = testdir.parseconfigure(x)
         col = testdir.getnode(config, x)
         assert isinstance(col, pytest.Module)
-        assert col.name == 'subdir/x.py'
+        assert col.name == 'x.py'
         assert col.parent.parent is None
         for col in col.listchain():
             assert col.config is config
@@ -528,6 +528,30 @@ class Test_genitems:
         assert s.endswith("test_example_items1.testone")
         print(s)
 
+    def test_class_and_functions_discovery_using_glob(self, testdir):
+        """
+        tests that python_classes and python_functions config options work
+        as prefixes and glob-like patterns (issue #600).
+        """
+        testdir.makeini("""
+            [pytest]
+            python_classes = *Suite Test
+            python_functions = *_test test
+        """)
+        p = testdir.makepyfile('''
+            class MyTestSuite:
+                def x_test(self):
+                    pass
+
+            class TestCase:
+                def test_y(self):
+                    pass
+        ''')
+        items, reprec = testdir.inline_genitems(p)
+        ids = [x.getmodpath() for x in items]
+        assert ids == ['MyTestSuite.x_test', 'TestCase.test_y']
+
+
 def test_matchnodes_two_collections_same_file(testdir):
     testdir.makeconftest("""
         import pytest
