@@ -149,7 +149,7 @@ class TestBootstrapping:
         mod.pytest_plugins = "pytest_a"
         aplugin = testdir.makepyfile(pytest_a="#")
         pluginmanager = get_plugin_manager()
-        reprec = testdir.getreportrecorder(pluginmanager)
+        reprec = testdir.make_hook_recorder(pluginmanager)
         #syspath.prepend(aplugin.dirpath())
         py.std.sys.path.insert(0, str(aplugin.dirpath()))
         pluginmanager.consider_module(mod)
@@ -184,8 +184,6 @@ class TestBootstrapping:
         assert pp.getplugin('hello') == a2
         pp.unregister(a1)
         assert not pp.isregistered(a1)
-        pp.unregister(name="hello")
-        assert not pp.isregistered(a2)
 
     def test_pm_ordering(self):
         pp = PluginManager()
@@ -274,14 +272,15 @@ class TestBootstrapping:
                 saveindent.append(pm.trace.root.indent)
                 raise ValueError(42)
         l = []
-        pm.trace.root.setwriter(l.append)
+        pm.set_tracing(l.append)
         indent = pm.trace.root.indent
         p = api1()
         pm.register(p)
 
         assert pm.trace.root.indent == indent
-        assert len(l) == 1
+        assert len(l) == 2
         assert 'pytest_plugin_registered' in l[0]
+        assert 'finish' in l[1]
         pytest.raises(ValueError, lambda: pm.register(api1()))
         assert pm.trace.root.indent == indent
         assert saveindent[0] > indent
@@ -405,11 +404,7 @@ class TestPytestPluginInteractions:
         pluginmanager.register(p3)
         methods = pluginmanager.listattr('m')
         assert methods == [p2.m, p3.m, p1.m]
-        # listattr keeps a cache and deleting
-        # a function attribute requires clearing it
-        pluginmanager._listattrcache.clear()
         del P1.m.__dict__['tryfirst']
-
         pytest.mark.trylast(getattr(P2.m, 'im_func', P2.m))
         methods = pluginmanager.listattr('m')
         assert methods == [p2.m, p1.m, p3.m]
@@ -436,6 +431,11 @@ def test_varnames():
     assert varnames(A().f) == ('y',)
     assert varnames(B()) == ('z',)
 
+def test_varnames_default():
+    def f(x, y=3):
+        pass
+    assert varnames(f) == ("x",)
+
 def test_varnames_class():
     class C:
         def __init__(self, x):
@@ -494,12 +494,10 @@ class TestMultiCall:
             return x + z
         reslist = MultiCall([f], dict(x=23, y=24)).execute()
         assert reslist == [24]
-        reslist = MultiCall([f], dict(x=23, z=2)).execute()
-        assert reslist == [25]
 
     def test_tags_call_error(self):
         multicall = MultiCall([lambda x: x], {})
-        pytest.raises(TypeError, multicall.execute)
+        pytest.raises(KeyError, multicall.execute)
 
     def test_call_subexecute(self):
         def m(__multicall__):
@@ -556,7 +554,7 @@ class TestMultiCall:
             l.append("m2 finish")
         m2.hookwrapper = True
         res = MultiCall([m2, m1], {}).execute()
-        assert res == [1, 2]
+        assert res == []
         assert l == ["m1 init", "m2 init", "m2 finish", "m1 finish"]
 
     def test_listattr_hookwrapper_ordering(self):
@@ -594,10 +592,8 @@ class TestMultiCall:
         m1.hookwrapper = True
 
         mc = MultiCall([m1], {})
-        with pytest.raises(mc.WrongHookWrapper) as ex:
+        with pytest.raises(TypeError):
             mc.execute()
-        assert ex.value.func == m1
-        assert ex.value.message
 
     def test_hookwrapper_too_many_yield(self):
         def m1():
@@ -606,29 +602,58 @@ class TestMultiCall:
         m1.hookwrapper = True
 
         mc = MultiCall([m1], {})
-        with pytest.raises(mc.WrongHookWrapper) as ex:
+        with pytest.raises(RuntimeError) as ex:
             mc.execute()
-        assert ex.value.func == m1
-        assert ex.value.message
+        assert "m1" in str(ex.value)
+        assert "test_core.py:" in str(ex.value)
+
+    @pytest.mark.parametrize("exc", [ValueError, SystemExit])
+    def test_hookwrapper_exception(self, exc):
+        l = []
+        def m1():
+            l.append("m1 init")
+            yield None
+            l.append("m1 finish")
+        m1.hookwrapper = True
+
+        def m2():
+            raise exc
+        with pytest.raises(exc):
+            MultiCall([m2, m1], {}).execute()
+        assert l == ["m1 init", "m1 finish"]
 
 
 class TestHookRelay:
     def test_happypath(self):
-        pm = PluginManager()
         class Api:
             def hello(self, arg):
                 "api hook 1"
-
-        mcm = HookRelay(hookspecs=Api, pm=pm, prefix="he")
-        assert hasattr(mcm, 'hello')
-        assert repr(mcm.hello).find("hello") != -1
+        pm = PluginManager([Api], prefix="he")
+        hook = pm.hook
+        assert hasattr(hook, 'hello')
+        assert repr(hook.hello).find("hello") != -1
         class Plugin:
             def hello(self, arg):
                 return arg + 1
-        pm.register(Plugin())
-        l = mcm.hello(arg=3)
+        plugin = Plugin()
+        pm.register(plugin)
+        l = hook.hello(arg=3)
         assert l == [4]
-        assert not hasattr(mcm, 'world')
+        assert not hasattr(hook, 'world')
+        pm.unregister(plugin)
+        assert hook.hello(arg=3) == []
+
+    def test_argmismatch(self):
+        class Api:
+            def hello(self, arg):
+                "api hook 1"
+        pm = PluginManager(Api, prefix="he")
+        class Plugin:
+            def hello(self, argwrong):
+                return arg + 1
+        with pytest.raises(PluginValidationError) as exc:
+            pm.register(Plugin())
+        assert "argwrong" in str(exc.value)
 
     def test_only_kwargs(self):
         pm = PluginManager()
@@ -639,18 +664,16 @@ class TestHookRelay:
         pytest.raises(TypeError, lambda: mcm.hello(3))
 
     def test_firstresult_definition(self):
-        pm = PluginManager()
         class Api:
             def hello(self, arg):
                 "api hook 1"
             hello.firstresult = True
-
-        mcm = HookRelay(hookspecs=Api, pm=pm, prefix="he")
+        pm = PluginManager([Api], "he")
         class Plugin:
             def hello(self, arg):
                 return arg + 1
         pm.register(Plugin())
-        res = mcm.hello(arg=3)
+        res = pm.hook.hello(arg=3)
         assert res == 4
 
 class TestTracer:
@@ -754,3 +777,105 @@ def test_importplugin_issue375(testdir):
     assert "qwe" not in str(excinfo.value)
     assert "aaaa" in str(excinfo.value)
 
+class TestWrapMethod:
+    def test_basic_happypath(self):
+        class A:
+            def f(self):
+                return "A.f"
+
+        l = []
+        def f(self):
+            l.append(1)
+            box = yield
+            assert box.result == "A.f"
+            l.append(2)
+        undo = add_method_wrapper(A, f)
+
+        assert A().f() == "A.f"
+        assert l == [1,2]
+        undo()
+        l[:] = []
+        assert A().f() == "A.f"
+        assert l == []
+
+    def test_no_yield(self):
+        class A:
+            def method(self):
+                return
+
+        def method(self):
+            if 0:
+                yield
+
+        add_method_wrapper(A, method)
+        with pytest.raises(RuntimeError) as excinfo:
+            A().method()
+
+        assert "method" in str(excinfo.value)
+        assert "did not yield" in str(excinfo.value)
+
+    def test_method_raises(self):
+        class A:
+            def error(self, val):
+                raise ValueError(val)
+
+        l = []
+        def error(self, val):
+            l.append(val)
+            yield
+            l.append(None)
+
+        undo = add_method_wrapper(A, error)
+
+        with pytest.raises(ValueError):
+            A().error(42)
+        assert l == [42, None]
+        undo()
+        l[:] = []
+        with pytest.raises(ValueError):
+            A().error(42)
+        assert l == []
+
+    def test_controller_swallows_method_raises(self):
+        class A:
+            def error(self, val):
+                raise ValueError(val)
+
+        def error(self, val):
+            box = yield
+            box.force_result(2)
+
+        add_method_wrapper(A, error)
+        assert A().error(42) == 2
+
+    def test_reraise_on_controller_StopIteration(self):
+        class A:
+            def error(self, val):
+                raise ValueError(val)
+
+        def error(self, val):
+            try:
+                yield
+            except ValueError:
+                pass
+
+        add_method_wrapper(A, error)
+        with pytest.raises(ValueError):
+            A().error(42)
+
+    @pytest.mark.xfail(reason="if needed later")
+    def test_modify_call_args(self):
+        class A:
+            def error(self, val1, val2):
+                raise ValueError(val1+val2)
+
+        l = []
+        def error(self):
+            box = yield (1,), {'val2': 2}
+            assert box.excinfo[1].args == (3,)
+            l.append(1)
+
+        add_method_wrapper(A, error)
+        with pytest.raises(ValueError):
+            A().error()
+        assert l == [1]
