@@ -1,11 +1,6 @@
 from _pytest.doctest import DoctestItem, DoctestModule, DoctestTextfile
 import py, pytest
 
-import pdb
-
-xfail_if_pdbpp_installed = pytest.mark.xfail(hasattr(pdb, "__author__"),
-    reason="doctest/pdbpp problem: https://bitbucket.org/antocuni/pdb/issue/24/doctests-fail-when-pdbpp-is-installed", run=False)
-
 class TestDoctests:
 
     def test_collect_testtextfile(self, testdir):
@@ -161,7 +156,6 @@ class TestDoctests:
         reprec = testdir.inline_run(p, "--doctest-modules")
         reprec.assertoutcome(failed=1)
 
-    @xfail_if_pdbpp_installed
     def test_doctestmodule_external_and_issue116(self, testdir):
         p = testdir.mkpydir("hello")
         p.join("__init__.py").write(py.code.Source("""
@@ -201,7 +195,6 @@ class TestDoctests:
             "*test_txtfile_failing.txt:2: DocTestFailure"
         ])
 
-    @xfail_if_pdbpp_installed
     def test_txtfile_with_fixtures(self, testdir):
         p = testdir.maketxtfile("""
             >>> dir = getfixture('tmpdir')
@@ -211,7 +204,6 @@ class TestDoctests:
         reprec = testdir.inline_run(p, )
         reprec.assertoutcome(passed=1)
 
-    @xfail_if_pdbpp_installed
     def test_txtfile_with_usefixtures_in_ini(self, testdir):
         testdir.makeini("""
             [pytest]
@@ -232,7 +224,6 @@ class TestDoctests:
         reprec = testdir.inline_run(p, )
         reprec.assertoutcome(passed=1)
 
-    @xfail_if_pdbpp_installed
     def test_doctestmodule_with_fixtures(self, testdir):
         p = testdir.makepyfile("""
             '''
@@ -244,7 +235,6 @@ class TestDoctests:
         reprec = testdir.inline_run(p, "--doctest-modules")
         reprec.assertoutcome(passed=1)
 
-    @xfail_if_pdbpp_installed
     def test_doctestmodule_three_tests(self, testdir):
         p = testdir.makepyfile("""
             '''
@@ -270,7 +260,6 @@ class TestDoctests:
         reprec = testdir.inline_run(p, "--doctest-modules")
         reprec.assertoutcome(passed=3)
 
-    @xfail_if_pdbpp_installed
     def test_doctestmodule_two_tests_one_fail(self, testdir):
         p = testdir.makepyfile("""
             class MyClass:
@@ -289,3 +278,95 @@ class TestDoctests:
         """)
         reprec = testdir.inline_run(p, "--doctest-modules")
         reprec.assertoutcome(failed=1, passed=1)
+
+    def test_ignored_whitespace(self, testdir):
+        testdir.makeini("""
+            [pytest]
+            doctest_optionflags = ELLIPSIS NORMALIZE_WHITESPACE
+        """)
+        p = testdir.makepyfile("""
+            class MyClass:
+                '''
+                >>> a = "foo    "
+                >>> print(a)
+                foo
+                '''
+                pass
+        """)
+        reprec = testdir.inline_run(p, "--doctest-modules")
+        reprec.assertoutcome(passed=1)
+
+    def test_non_ignored_whitespace(self, testdir):
+        testdir.makeini("""
+            [pytest]
+            doctest_optionflags = ELLIPSIS
+        """)
+        p = testdir.makepyfile("""
+            class MyClass:
+                '''
+                >>> a = "foo    "
+                >>> print(a)
+                foo
+                '''
+                pass
+        """)
+        reprec = testdir.inline_run(p, "--doctest-modules")
+        reprec.assertoutcome(failed=1, passed=0)
+
+    def test_ignored_whitespace_glob(self, testdir):
+        testdir.makeini("""
+            [pytest]
+            doctest_optionflags = ELLIPSIS NORMALIZE_WHITESPACE
+        """)
+        p = testdir.maketxtfile(xdoc="""
+            >>> a = "foo    "
+            >>> print(a)
+            foo
+        """)
+        reprec = testdir.inline_run(p, "--doctest-glob=x*.txt")
+        reprec.assertoutcome(passed=1)
+
+    def test_non_ignored_whitespace_glob(self, testdir):
+        testdir.makeini("""
+            [pytest]
+            doctest_optionflags = ELLIPSIS
+        """)
+        p = testdir.maketxtfile(xdoc="""
+            >>> a = "foo    "
+            >>> print(a)
+            foo
+        """)
+        reprec = testdir.inline_run(p, "--doctest-glob=x*.txt")
+        reprec.assertoutcome(failed=1, passed=0)
+
+    def test_ignore_import_errors_on_doctest(self, testdir):
+        p = testdir.makepyfile("""
+            import asdf
+
+            def add_one(x):
+                '''
+                >>> add_one(1)
+                2
+                '''
+                return x + 1
+        """)
+
+        reprec = testdir.inline_run(p, "--doctest-modules",
+                                    "--doctest-ignore-import-errors")
+        reprec.assertoutcome(skipped=1, failed=1, passed=0)
+
+    def test_junit_report_for_doctest(self, testdir):
+        """
+        #713: Fix --junit-xml option when used with --doctest-modules.
+        """
+        p = testdir.makepyfile("""
+            def foo():
+                '''
+                >>> 1 + 1
+                3
+                '''
+                pass
+        """)
+        reprec = testdir.inline_run(p, "--doctest-modules",
+                                    "--junit-xml=junit.xml")
+        reprec.assertoutcome(failed=1)
